﻿/***
 *** 
 *** Author: Steve Rysavy, dynameomics_info@uw.edu
 ***         University of Washington
 ***
 *** Copyright: 2014, All rights reserved.
 *** NOTICE: Unauthorized use, duplication and / or distribution
 ***         will be subject to criminal and civil penalties.
 ***
 ***/

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net;
using System.IO;

namespace FragmentSearchServiceExample
{
    class Program
    {
        static readonly string kURLInternetPDB = @"http://www.pdb.org/pdb/download/downloadFile.do?fileFormat=pdb&compression=NO&structureId=";
        static readonly string kPDBCode = @"1enh";
        
        static void Main(string[] args)
        {
            //Create object reference to fragment search service
            FragmentWebServiceReference.FragmentSearchService webService = new FragmentWebServiceReference.FragmentSearchService();
            
            //Create list to hold fragment results
            List<FragmentWebServiceReference.ICSortResult> fragmentResults = new List<FragmentWebServiceReference.ICSortResult>();

            //Get the contents of a PDB file
            Console.WriteLine("Getting PDB contents for {0}...", kPDBCode);
            byte[] pdbFileContents = GetPDBFile(kPDBCode);

            try
            {
                Console.WriteLine("Initiating fragment search...");
                //Execute fragments search using web service
                fragmentResults = webService.GetFragmentsFromICSearch(
                    pdbFileContents     //byte array of PDB file contents
                    , "A"               //Chain identifier
                    , 0                 //Model identifier (1 if only one model exists)
                    , 23                //First residue number of missing residues
                    , 27                //Last residue number of missing residues
                    , 5                 //Total number of missing residues
                    , "1enh"            //Comma-separated list of PDB structures to exclude from search results
                    , 10                //Maximum number of results for search to return
                    , 1.5f              //Maximum variance of internal coordinate distance between atom pairs
                    , 1).ToList();      //ID of fragment library to search
            }
            catch(Exception  e)
            {
                Console.WriteLine("Could not initiate fragment search. Error: {0}", e.Message);
            }

            //Print out all fragment search results to console
            Console.WriteLine("Listing all search results...");
            foreach (FragmentWebServiceReference.ICSortResult result in fragmentResults)
            {
                Console.WriteLine("Search GUID: {0}\tIC RMSD: {1}", result.icSortResultGuid, result.icRMSD);
            }

            //Create structure to hold best fragment result information
            FragmentWebServiceReference.ICSortResult bestFragmentResult;

            //Create string to hold best fragment result PDB contents
            string fragmentPDBContents;

            //Check if fragment matches were found
            if (fragmentResults.Count > 0)
            {
                //Get best matching fragment result
                bestFragmentResult = fragmentResults.First();

                try
                {
                    Console.WriteLine("Press a key to print PDB contents for top fragment result.");
                    Console.ReadKey();
                    Console.WriteLine("Retreiving top fragment result PDB contents...");

                    //Get PDB contents for best result
                    fragmentPDBContents = webService.GetPDBFileFromICSearchGuid(bestFragmentResult.icSortResultGuid);

                    //Write contents to console
                    Console.WriteLine(fragmentPDBContents);
                }
                catch
                {
                    Console.WriteLine("Could not retrieve PDB contents for best fragment result.");
                }
            }


            Console.WriteLine("Finished example.");
            Console.WriteLine("Press any key to exit.");
            Console.Read();
        }

        /// <summary>
        /// Retrieves PDB file from PDB repository for given PDB code
        /// </summary>
        /// <param name="pdb4">PDB code of protein structure to retrieve</param>
        /// <returns>PDB file contents</returns>
        static byte[] GetPDBFile(string pdb4)
        {
            byte[] pdbFileArray;
            WebClient wc = new WebClient();
            
            try
            {
                //Retrieves PDB file from the RCSB Protein Data Bank
                pdbFileArray = wc.DownloadData(String.Format("{0}{1}", kURLInternetPDB, pdb4));
            }
            catch (Exception e)
            {
                throw new Exception(String.Format("PDB does not exist or cannot find file. Error {0}", e.Message));
            }
            finally
            {
                if (wc != null) { wc.Dispose(); }
            }

            return pdbFileArray;
        }
    }
}
